#! /usr/bin/python3.9

"""
Mahyar@Mahyar24.com, Thu 19 Aug 2021.
"""

from datetime import timedelta

def match(
    base: list[tuple[timedelta, timedelta]], other: list[tuple[timedelta, timedelta]]
) -> float: ...
